require_relative 'ns_macro_processor/processor_error'
require_relative 'ns_macro_processor/syntax_error'
require_relative 'ns_macro_processor/tokens'

module NsMacroProcessor

  class Processor

    def initialize(params)
      @params = params
    end

    def parse(text)
      @tokens = Tokens.new(text)
      @tokens.advance
      result = words
      expect(Tokens::EOF)
      result
    end

    private

    def words
      result = []
      while @tokens.peek != Tokens::EOF
        result << word
      end
      result.join
    end

    def word
      result = @tokens.peek
      @tokens.advance
      if result == '~' and accept('{')
        @tokens.push_back(macro)
        result = word
      end
      result
    end

    def macro
      args = []
      loop do
        args << word
        break if @tokens.peek == '}' || @tokens.peek == Tokens::EOF
      end
      expect('}')
      execute(args.join.split(' '))
    end

    def execute(args)
      case args[0]
      when 'define'
        raise SyntaxError.new('~{define name value}', args.join(' ')) unless args.length >= 2
        @params[args[1]] = args[2..-1].join(' ')
        ''
      when 'include'
        lookup(:filestore).contents_of(args[1])
      else
        lookup(args[0])
      end
    end

    def lookup(field_name)
      value = @params[field_name] || @params[field_name.to_sym]
      raise ProcessorError.new(field_name) unless value
      value
    end

    def accept(s)
      if (@tokens.peek == s)
        @tokens.advance
        true
      else
        false
      end
    end

    def expect(s)
      return true if accept(s)
      raise SyntaxError.new(s, @tokens.peek)
      false
    end

  end

end

